//
// Created by dawsh on 6/16/24.
//
#include <jtest/jtest.hpp>
namespace jtest {

    // Globals for test tracking
    std::vector<testdef> testlist;
    int rantests;
    int passedtests;
    int failedtests;

    void log(std::vector<jlog::token> tokens)
    {
        std::vector<jlog::token> wtokens;
        auto head = jlog::token{.colorCode = jlog::ansi_escape_codes::FG_WHITE, .content ="JTEST"};

        wtokens.push_back(head);
        wtokens.insert(wtokens.end(), tokens.begin(), tokens.end());

        jlog::log(wtokens, "test_results.log");
    }

    std::vector<jlog::token> log_test_format(const std::string &testname, const std::string &file, int line,
        bool passed) {
        std::vector<jlog::token> wtokens;

        auto filedata = jlog::token{.content = std::format("{}:{}", file, line)};

        std::vector<jlog::token> teststate;
        if (passed)
        {
            teststate.push_back(jlog::token{.colorCode = jlog::ansi_escape_codes::FG_GREEN, .content = testname});
            teststate.push_back(jlog::token{.content = "Passed:", .delimiter = ""});
        } else
        {
            teststate.push_back(jlog::token{.colorCode = jlog::ansi_escape_codes::FG_RED, .content = testname});
            teststate.push_back(jlog::token{.content = "Failed:", .delimiter = ""});
        }

        auto raninfo = jlog::token{.content = std::format("{}/{}", rantests, testlist.size())};

        wtokens.push_back(filedata);
        wtokens.insert(wtokens.end(), teststate.begin(), teststate.end());
        wtokens.push_back(raninfo);

        return wtokens;
    }

    std::vector<jlog::token> log_test_tracking_format() {
        auto ran = jlog::token{.content = std::format("Tests Ran: [{}/{}]", rantests, testlist.size()), .delimiter = ""};
        auto failed = jlog::token{.content = std::format("Failed: [{}/{}]", failedtests, rantests), .delimiter = ""};
        auto passed = jlog::token{.content = std::format("Passed: [{}/{}]", passedtests, rantests), .delimiter = ""};

        return {ran, failed, passed};
    }

    void definetest(const std::string &testname, const std::function<void()> &callback, const std::string &file,
        int line) {
        testlist.push_back(testdef(testname, callback, file, line));
    }

    bool check(bool condition) {
        if (!condition)
            throw std::runtime_error("Test check failed!!");
        return condition;
    }

    bool check_float_eq(float a, float b, float epsilon) {
        if (std::abs(a - b) > epsilon) {
            throw std::runtime_error ("Test check failed!!");
        }
        return true;
    }

    bool check_double_eq(double a, double b, double epsilon) {
        if (std::abs(a - b) > epsilon) {
            throw std::runtime_error ("Test check failed!!");
        }
        return true;
    }

    bool check_string_eq(const std::string &a, const std::string &b) {
        if (a == b) // This is valid and recommended C++2x.
            throw std::runtime_error("Test check failed!!");
        return true;
    }

    bool test(const std::string &testname, const std::function<void()> &callback, const std::string &file, int line) {
        bool passed = true;

        try { callback(); }
        catch(const std::exception& e)
        { passed = false; }
        catch(...) // <- Basically covers all exception cases. GTest does something similar
        { passed = false; }

        rantests++;

        if (passed) { passedtests++; }
        else        { failedtests++; }

        jtest::log(log_test_format(testname, file, line, passed));

        return passed;

    }

    void run_tests() {

        for (testdef& td : testlist)
        {
            td.passed = test(td.testname, td.callback, td.file, td.line);
        }

        jtest::log(log_test_tracking_format());
    }
}


